from dune.perftool.error import PerftoolLoopyError
from dune.perftool.generation import (get_counted_variable,
                                      kernel_cached,
                                      temporary_variable,
                                      )


class FlipFlopBuffer(object):
    def __init__(self, identifier):
        self.identifier = identifier

        # Initialize the counter that switches between the base storages!
        self._current = 0

        # Generate the base storage names
        self.base_storage = tuple("{}_base_{}".format(self.identifier, i) for i in (0, 1))

    def switch_base_storage(self):
        self._current = (self._current + 1) % 2

    def get_temporary(self, **kwargs):
        assert("base_storage" not in kwargs)
        assert("storage_shape" not in kwargs)

        # Select the base storage and increase counter
        base = self.base_storage[self._current]

        # Construct a temporary name
        name = kwargs.pop("name", None)
        if name is None:
            name = get_counted_variable(self.identifier)

        # Construct the temporary and return it
        temporary_variable(name,
                           base_storage=base,
                           managed=True,
                           **kwargs
                           )

        return name


@kernel_cached
def initialize_buffer(identifier):
    return FlipFlopBuffer(identifier)


def get_buffer_temporary(identifier, **kwargs):
    return initialize_buffer(identifier).get_temporary(**kwargs)


def switch_base_storage(identifier):
    initialize_buffer(identifier).switch_base_storage()
